
FUNCTION p_reloadHyperbolic(z, path_dir, z_lastRev, z_1controlDeg, p_lastRev, p_1controlDeg, k0, w_soil)
IMPLICIT NONE

	INTEGER, INTENT(IN) :: path_dir	
	DOUBLE PRECISION, INTENT(IN) :: z, z_lastRev, z_1controlDeg, p_lastRev, p_1controlDeg, k0, w_soil	
	DOUBLE PRECISION ::  z0, zm, b, xi_controlP, p_reloadHyperbolic
	
	b = (z_1controlDeg - z_lastRev)/(p_1controlDeg - p_lastRev)- 1.0d0/k0
	xi_controlP = (z_1controlDeg - z_lastRev)/(p_1controlDeg*(1+w_soil)*b) 
	p_reloadHyperbolic =  p_lastRev + (z - z_lastRev) / (1.0d0/k0 + path_dir*(z-z_lastRev)/((1+w_soil)*p_1controlDeg*xi_controlP)) 
	
END FUNCTION p_reloadHyperbolic

FUNCTION kz_reloadHyperbolic(z, path_dir, z_lastRev, z_1controlDeg, p_lastRev, p_1controlDeg, k0, w_soil)
IMPLICIT NONE

	INTEGER, INTENT(IN) :: path_dir	
	DOUBLE PRECISION, INTENT(IN) :: z, z_lastRev, z_1controlDeg, p_lastRev, p_1controlDeg, k0, w_soil
	DOUBLE PRECISION ::  z0, zm, a, b, xi_controlP, kz_reloadHyperbolic

	b = (z_1controlDeg - z_lastRev)/(p_1controlDeg - p_lastRev)- 1.0d0/k0
	xi_controlP = (z_1controlDeg - z_lastRev)/(p_1controlDeg*(1+w_soil)*b) 
	a = 1.0d0/k0 + path_dir*(z-z_lastRev)/((1+w_soil)*p_1controlDeg*xi_controlP)
    kz_reloadHyperbolic = 1.0d0/a + path_dir*(z_lastRev -z)/((1+w_soil) *p_1controlDeg*xi_controlP*a**2)  

END FUNCTION kz_reloadHyperbolic

!-----------------------------------------------------------------------------------------------------
FUNCTION p_unloadHyperbolic(z, path_dir, z1, p1, p2, k0, w_soil)
	IMPLICIT NONE	
	
	INTEGER, INTENT(IN) :: path_dir
	DOUBLE PRECISION, INTENT(IN) :: z, z1, p1,  p2
	DOUBLE PRECISION :: k0, w_soil, p_unloadHyperbolic
	
	p_unloadHyperbolic = p1 + (z-z1)/(1.0d0/k0 + path_dir*(z-z1)/((1+w_soil)*p2))
	
END FUNCTION p_unloadHyperbolic

FUNCTION kz_unloadHyperbolic(z, path_dir, z1, p2, k0, w_soil)
	IMPLICIT NONE	
	
	INTEGER, INTENT(IN) :: path_dir
	DOUBLE PRECISION, INTENT(IN) :: z, z1, p2, k0, w_soil
	DOUBLE PRECISION ::  A, kz_unloadHyperbolic
	
	A = 1.0d0/k0 + path_dir*(z-z1)/((1+w_soil)*p2)
    kz_unloadHyperbolic = 1.0d0/A+ path_dir*(z1-z)/((1+w_soil)*p2*A**2)
   	
END FUNCTION kz_unloadHyperbolic

!-----------------------------------------------------------------------------------------------------
FUNCTION p_reloadcubic(z, z1, z2, p1, p2)
	IMPLICIT NONE
	
	DOUBLE PRECISION, INTENT(IN) :: z
	DOUBLE PRECISION, INTENT(IN) :: z1, z2, p1, p2	
	DOUBLE PRECISION :: z0, zm, p_reloadcubic 
	
	z0 = (z1+z2)/2.0d0
    zm = (z1-z2)/2.0d0
    p_reloadcubic = (p1+p2)/2.0d0 +(p1-p2)/4.0d0*(3.0d0*(z-z0)/zm - ((z-z0)/zm)**3)
	
END FUNCTION p_reloadcubic 

FUNCTION kz_reloadcubic(z, z1, z2, p1, p2)
	IMPLICIT NONE
	
	DOUBLE PRECISION, INTENT(IN) :: z
	DOUBLE PRECISION, INTENT(IN) ::  z1, z2, p1, p2	
	DOUBLE PRECISION ::  z0, zm, kz_reloadcubic 
	
	z0 = (z1+z2)/2.0d0
    zm = (z1-z2)/2.0d0
    kz_reloadcubic = (p1-p2)/4.0d0*(3.0d0/zm - 3.0d0*(z-z0)**2.0d0/zm**3) 
		
END FUNCTION kz_reloadcubic 

!-----------------------------------------------------------------------------------------------------
FUNCTION iterationCubic(z_1controlDeg, z_lastRev, p_1controlDeg, p_lastRev, p_1bound) 
	IMPLICIT NONE
	
	DOUBLE PRECISION, INTENT(IN) :: z_1controlDeg, z_lastRev, p_1controlDeg, p_lastRev, p_1bound
	DOUBLE PRECISION :: iterationCubic,  error, p, p_reloadcubic, z
	
	error = -1
	z = z_lastRev
	DO while (error <= 0)
		z = z + 0.000001
		p = p_reloadcubic(z, z_1controlDeg, z_lastRev, p_1controlDeg, p_lastRev)
		error = p - p_1bound
	END DO
	iterationCubic = z	
	
END FUNCTION iterationCubic 

!-----------------------------------------------------------------------------------------------------
FUNCTION iterationHyperbolic(z_1controlDeg, z_lastRev, p_1controlDeg, p_lastRev, p_1bound, k0, w_soil)
	IMPLICIT NONE
	
	DOUBLE PRECISION, INTENT(IN) :: z_1controlDeg, z_lastRev, p_1controlDeg, p_lastRev, p_1bound, k0, w_soil
	DOUBLE PRECISION :: iterationHyperbolic, error, p, p_reloadHyperbolic, z
	
	error = -1.0d0
	z = z_lastRev
	write(*,*) 'hei2 p_1bound', p_1bound
	write(*,*) 'z_lastrev z_1controlDeg, p_lastRev, p_1controlDeg', z_lastrev, z_1controlDeg, p_lastRev, p_1controlDeg
	DO while (error <= 0.0d0)
		z = z + 0.01
		p = p_reloadHyperbolic(z, 1, z_lastRev, z_1controlDeg, p_lastRev, p_1controlDeg, k0, w_soil)
		error = p - p_1bound
		
	END DO
	 iterationHyperbolic = z
		
END FUNCTION iterationHyperbolic 

